import React, { useState } from 'react';
import { View, Text, Image, StyleSheet, TouchableOpacity } from 'react-native';
import ActionSheet, { SheetManager } from 'react-native-actions-sheet';
import { Colors } from 'src/theme';
import { API_HOST } from 'src/constants';
import { getFontSize } from 'src/utils';
import { useSafeAreaInsets } from 'react-native-safe-area-context';
import { ChatProps } from '../types';
import { useNavigation } from '@react-navigation/native';
import { NAVIGATION_PAGES } from 'src/types';
import { usePostSetBlockMutation, usePostSetMuteMutation } from '@api/chat';
import { useChatStore } from 'src/stores/chatStore';
import TrashIcon from 'assets/icons/travels-screens/trash-solid.svg';
import BanIcon from 'assets/icons/messages/ban.svg';
import BellSlashIcon from 'assets/icons/messages/bell-slash.svg';

const MoreModal = () => {
  const insets = useSafeAreaInsets();
  const navigation = useNavigation();
  const { setIsWarningModalVisible } = useChatStore();

  const [chatData, setChatData] = useState<
    | (ChatProps & {
        token: string;
        refetch: () => void;
      })
    | null
  >(null);
  const { mutateAsync: muteUser } = usePostSetMuteMutation();
  const { mutateAsync: blockUser } = usePostSetBlockMutation();
  const [shouldOpenWarningModal, setShouldOpenWarningModal] = useState(false);

  const handleSheetOpen = (
    payload:
      | (ChatProps & {
          token: string;
          refetch: () => void;
        })
      | null
  ) => {
    setChatData(payload);
  };

  const handleMute = async () => {
    if (!chatData) return;

    await muteUser(
      {
        token: chatData.token,
        value: chatData.muted === 1 ? 0 : 1,
        conversation_with_user: chatData.uid
      },
      {
        onSuccess: () => {
          setChatData({ ...chatData, muted: chatData.muted === 1 ? 0 : 1 });
        }
      }
    );
    chatData.refetch();
  };

  const handleBlock = async () => {
    // if (!chatData) return;
    // await blockUser({
    //   value: chatData.blocked === 1 ? 0 : 1,
    //   conversation_with_user: chatData.uid
    // });
  };

  const handleDelete = async () => {
    setShouldOpenWarningModal(true);

    setTimeout(() => {
      SheetManager.hide('more-modal');
      setShouldOpenWarningModal(false);
    }, 300);
  };

  return (
    <ActionSheet
      id="more-modal"
      gestureEnabled={true}
      onBeforeShow={(sheetRef) => {
        const payload = sheetRef || null;
        handleSheetOpen(payload);
      }}
      onClose={() => {
        if (shouldOpenWarningModal) {
          setIsWarningModalVisible(true);
        }
      }}
      containerStyle={styles.sheetContainer}
      defaultOverlayOpacity={0.5}
      indicatorStyle={{ backgroundColor: 'transparent' }}
    >
      {chatData && (
        <View style={[styles.container, { paddingBottom: 8 + insets.bottom }]}>
          <TouchableOpacity
            style={styles.header}
            onPress={() => {
              SheetManager.hide('more-modal');
              navigation.navigate(
                ...([NAVIGATION_PAGES.PUBLIC_PROFILE_VIEW, { userId: chatData.uid }] as never)
              );
            }}
          >
            <Image source={{ uri: API_HOST + chatData?.avatar }} style={styles.avatar} />
            <Text style={styles.name}>{chatData.name}</Text>
          </TouchableOpacity>

          <View style={styles.optionsContainer}>
            <TouchableOpacity style={styles.option} onPress={handleMute}>
              <Text style={styles.optionText}>{chatData.muted === 1 ? 'Unmute' : 'Mute'}</Text>
              <BellSlashIcon fill={Colors.DARK_BLUE} />
            </TouchableOpacity>
          </View>

          <View style={[styles.optionsContainer, { paddingVertical: 0, gap: 0 }]}>
            <TouchableOpacity style={[styles.option, styles.dangerOption]}>
              <Text style={[styles.optionText, styles.dangerText]}>Block {chatData.name}</Text>
              <BanIcon fill={Colors.RED} />
            </TouchableOpacity>

            <TouchableOpacity style={[styles.option, styles.dangerOption]} onPress={handleDelete}>
              <Text style={[styles.optionText, styles.dangerText]}>Delete chat</Text>
              <TrashIcon fill={Colors.RED} width={18} height={18} />
            </TouchableOpacity>
          </View>
        </View>
      )}
    </ActionSheet>
  );
};

const styles = StyleSheet.create({
  sheetContainer: {
    borderTopLeftRadius: 15,
    borderTopRightRadius: 15
  },
  container: {
    backgroundColor: 'white',
    paddingHorizontal: 16,
    paddingTop: 8,
    gap: 16
  },
  header: {
    flexDirection: 'row',
    alignItems: 'center',
    borderBottomWidth: 1,
    borderBottomColor: Colors.FILL_LIGHT,
    gap: 8
  },
  avatar: {
    width: 32,
    height: 32,
    borderRadius: 16,
    borderWidth: 1,
    borderColor: Colors.FILL_LIGHT
  },
  name: {
    fontSize: getFontSize(14),
    fontFamily: 'montserrat-700',
    color: Colors.DARK_BLUE
  },
  optionsContainer: {
    paddingVertical: 10,
    paddingHorizontal: 8,
    gap: 16,
    borderRadius: 8,
    backgroundColor: Colors.FILL_LIGHT
  },
  option: {
    flexDirection: 'row',
    alignItems: 'center',
    justifyContent: 'space-between'
  },
  optionText: {
    fontSize: getFontSize(12),
    fontWeight: '600',
    color: Colors.DARK_BLUE
  },
  dangerOption: {
    paddingVertical: 10,
    borderBottomWidth: 1,
    borderBlockColor: Colors.WHITE
  },
  dangerText: {
    color: Colors.RED
  }
});

export default MoreModal;
